import glob,os,json
from abstract_utilities import *
from pathlib import Path
from io import BytesIO
from abstract_utilities.path_utils import get_files
from .serverManager import *
def get_server_mgr(src_dir=None,domain=None,imgs_dir=None,fb_id=None,directory=None):
    server_mgr = serverManager(src_dir=src_dir,domain=domain,imgs_dir=imgs_dir,fb_id=fb_id,directory=directory)
    return server_mgr
def get_abs_path():
    return os.path.abspath(__file__)
def get_abs_dir():
    abs_path = get_abs_path()
    return os.path.dirname(abs_path)
def get_sections_dir():
    return 'collect_media/sections'
def get_src_dir():
    return get_server_mgr().src_dir
def get_base_dir():
    return os.path.dirname(get_src_dir())
def get_imgs_dir():
    return get_server_mgr().imgs_dir
def get_json_directory():
    server_mgr = get_server_mgr()
    return server_mgr.json_dir
def get_json_directories():
    return get_dir_paths(get_json_directory())
def get_dir_paths(directory):
    json_dirs = [os.path.join(directory,item) for item in os.listdir(directory)]
    json_dirs = [json_dir for json_dir in json_dirs if os.path.isdir(json_dir)]
    return json_dirs
def get_data(file_path):
    return safe_read_from_json(file_path)
def consolidate_media(file_path):
    data = safe_read_from_json(file_path)
    data = get_new_media(data)
    safe_dump_to_file(data,file_path)
    return data
def get_json_data(file_path):
    json_contents = safe_read_from_json(file_path)
    if not json_contents:
        return 
    return json_contents
def get_all_json_paths(directory):
    json_paths = glob.glob(os.path.join(directory, '**', '*.json'), recursive=True)
    return json_paths
def combine_media(media,media_2):
    for key,value in media.items():
        media[key] = check_value(value) or check_value(media_2.get(key))
    return media
def find_all_files_with_substring(substr, directory=None):
    base_dir = Path(directory) if directory else Path.cwd()
    return [path for path in base_dir.rglob('*') if os.path.isfile(str(path)) and substr in os.path.basename(str(path))]

def create_source_dirs():
    json_directories = get_json_directories()
    for json_directory in json_directories:
        basename = os.path.basename(json_directory)
        filename,ext = os.path.splitext(basename)
        source_dir = os.path.join(get_sections_dir(),filename)
        os.makedirs(source_dir,exist_ok=True)
        all_medias = get_all_medias(json_directory)
        all_keywords = get_all_keywords(json_directory)
        sources_json = os.path.join(source_dir,'sources.json')
        if not os.path.isfile(sources_json):
            safe_dump_to_file([],sources_json)
        sources_data =safe_read_from_json(sources_json)
        images_json = os.path.join(source_dir,'images.json')
        if not os.path.isfile(sources_json):
            safe_dump_to_file([],images_json)    
        images_data =safe_read_from_json(images_json)
        keywords_json = os.path.join(source_dir,'keywords.json')
        if not os.path.isfile(keywords_json):
            safe_dump_to_file([],keywords_json)
        images_data,sources_data = consolidate_medias(all_medias,images=images_data,others=sources_data)
        safe_dump_to_file(sources_data,sources_json)
        safe_dump_to_file(images_data,images_json)



from .abstract_types import *
from .abstract_webserver import *
from .directory_processor import *
from .get_image_json import *
from .get_source import *
from .json_convention import *
from .path_utils import *
from .social_media_utils import *



import urllib.parse
from typing import *
from abstract_utilities import eatAll
DEFINITIION_ALIAS = {
    "text":{"alias":["text","post","content","tweet","thread","body"],"description":"The main text."},
    "url":{"alias":["url","u","link","site","domain","intentUrl","link_address","address","canonical",'path'],"description":"A URL to include in the post."},
    "via":{"alias":["via","username","user","@","uploader","author"],"description":"A username to attribute the post to."},
    "hashtags":{"alias":["hashtags","hashtag","tag","tags"],"description":"Comma-separated hashtags (without the # symbol)."},
    "related":{"alias":["related","contributor","credit","cc","bcc"],"description":"Comma-separated related accounts."},
    "subject":{"alias":["title","subject","heading","header"],"description":"The email subject."},
    "body":{"alias":["text","post","content","tweet","thread","body"],"description":"The email body."},
    "cc":{"alias":["related","contributor","credit","cc","bcc"],"description":"Additional email addresses."},
    "bcc":{"alias":["related","contributor","credit","cc","bcc"],"description":"Additional email addresses."}
}
def get_alias_params(string):
    return DEFINITIION_ALIAS.get(string)
SOCIAL_SHARE_PARAMS={
    "x":{
        "url":"https://twitter.com/intent/tweet",
        "params":{
            "text":get_alias_params("text"),
            "url":get_alias_params("url"),
            "via":get_alias_params("via"),
            "hashtags":get_alias_params("hashtags"),
            "related":get_alias_params("related")
            },
        "characters":{"limit":280,"optimal":100,"mobile_cutoff":150,"url_len":30},
        "alias":["x","twitter","x.com","tweet","twitter.com"],
        "hash_symbol":False
        },
     "facebook":{
         "url":"http://facebook.com/sharer.php",
         "params":{
             "u":get_alias_params("url")
             },
         "characters":{"limit":63206,"optimal":50,"mobile_cutoff":150,"url_len":None},
         "alias":["facebook","fb","facebook.com","meta","meta.com"],
         "hash_symbol":True
         },
    
     "threads":{
         "url":"https://www.threads.net/intent/post",
         "params":{
             "text":get_alias_params("text")
             },
         "characters":{"limit":500,"optimal":150,"mobile_cutoff":150,"url_len":None},
         "alias":["threads","@","threads.com","@.com"],
         "hash_symbol":True
         },
    "mailto":{
        "url":"mailto:",
        "params":{
             "subject":get_alias_params("subject"),
             "body":get_alias_params("body"),
             "cc":get_alias_params("cc"),
             "bcc":get_alias_params("bcc")
             },
        "characters":{"limit":None,"optimal":None,"mobile_cutoff":None},
        "alias":["mailto","mail","email","email.com","mail.com"],
        "hash_symbol":True
        },
    "minds":{
        "url":"https://www.minds.com/newsfeed/subscriptions/latest",
        "params":{
            "intentUrl":get_alias_params("url")
            },
        "characters":{"limit":500,"optimal":125,"mobile_cutoff":150,"url_len":None},
        "alias":["minds","mindscollective","collective"],
        "hash_symbol":True
        }
             
    }

def clean_var(var):
    var = str(var)
    return eatAll(var,['/',',','"',"'",'#','\t','\n'])
def capitalize_util(string):
    if not string:
        return string
    cap = string[0].upper()
    if len(string)>1:
        ital = string[1:].upper()
        cap = f"{cap}{ital}"
    return cap
def capitalize(string,replace_all=True):
    strings = string.replace('_',' ').replace('-',' ').replace(',',' ').split(' ')
    strings = [capitalize_util(string) for string in strings]
    strings = ''.join(strings)
    return strings
def clean_keyword(string,cleanIt=True,capitalizeIt=True,replace_all=True):
    if cleanIt:
        string = clean_var(string)
    if capitalizeIt:
        string = capitalize(string,replace_all=replace_all)
    return string
def generate_keywords_list(keywords,cleanIt=True,capitalizeIt=True,replace_all=True,hashtag=False):
    keywords = keywords or []
    if isinstance(keywords,tuple) or isinstance(keywords,set):
        keywords = list(keywords)
    if isinstance(keywords,str):
        keywords = keywords.replace('#',',').split(',')
    if isinstance(keywords,list):
        keywords = [clean_keyword(keyword,cleanIt=cleanIt,capitalizeIt=capitalizeIt,replace_all=replace_all) for keyword in keywords if keyword]
    return keywords
def convert_keywords_list_to_text(keywords,hash_symbol=True,cleanIt=True,capitalizeIt=True,replace_all=True,hashtag=False):
    keywords = generate_keywords_list(keywords,cleanIt=cleanIt,capitalizeIt=capitalizeIt,replace_all=replace_all,hashtag=hashtag)
    if isinstance(hash_symbol,bool):
        if hash_symbol:
            hash_symbol = ' #'
        else:
            hash_symbol = ','
    keywords = hash_symbol+hash_symbol.join(keywords)
    if keywords:
        keywords = eatAll(keywords,[' ',','])
    return keywords
def generate_hashtags(keywords,hash_symbol=True,cleanIt=True,capitalizeIt=True,replace_all=True) -> str:
    keywords = generate_keywords_list(
        keywords=keywords,
        cleanIt=cleanIt,
        capitalizeIt=capitalizeIt,
        replace_all=replace_all
        )

def encode_uri(string):
    if string == None:
        return None
    string = str(string)
    encoded_string = urllib.parse.quote(string)
    return encoded_string
def get_share_params(platform):
    params = SOCIAL_SHARE_PARAMS.get(platform)
    if params:
        return params
    platform_lower = platform.lower()
    first_key = None
    for key,values in SOCIAL_SHARE_PARAMS.items():
        alias = values.get("alias")
        if platform_lower in alias:
            return values
        if first_values == None:
            first_values=values
    return first_values
def get_precise_param(params,input_params):
    alias = params.get('alias')
    input_params_copy = input_params.copy()
    for key,value in input_params_copy.items():
        if key in alias:
            del input_params[key]
            return value,input_params
    return None,input_params
def create_output_text(input_params,hash_symbol):
    text_output = ''
    for key,value in input_params.items():
        if value:
            if key == 'text':
                text_output+=value
            elif key == 'via':
                if text_output:
                    text_output+='\n'
                if not value.startswith('@'):
                    value = f"@{value}"
                text_output+=value
            elif key == 'url':
                if text_output:
                    text_output+='\n'
                text_output+=value
            elif key == 'hashtags':
                if text_output:
                    text_output+='\n'
                value = get_max_hashtags(value,hash_symbol=hash_symbol)
                text_output+=value
    return text_output
def create_output_url(output_params,share_url):
    url = None
    for key,value in output_params.items():
        if value:
            value = encode_uri(value)
            if url == None:
                url = f"{share_url}?{key}={value}"
            else:
               url = f"{url}&{key}={value}"
    return url
def elipse_if_over(string,limit):
    string = str(string)
    if len(string)>limit:
        string = string[:limit-3]
        string = f"{string}..."
    return string
def get_exact_list(obj, items):
    return obj[:items] if len(obj) >= items else obj
def get_max_hashtags(hashtags:list,char_limit=None,hashtag_limit=None,hash_symbol=True):
    all_hashtags_text = convert_keywords_list_to_text(hashtags,hash_symbol=hash_symbol)
    hashtags = all_hashtags_text.split(' ')
    output_text = ''
    hashtag_count = 0
    char_limit = char_limit or len(str(hashtags))+1
    char_available = char_limit
    for hashtag in hashtags:
        hashtag_len = len(hashtag)
        if char_available>=hashtag_len:
            test_text=f"{output_text} {hashtag}"
            test_text = eatAll(test_text,[' '])
            if char_limit>=len(test_text):
                hashtag_count+=1
                output_text = test_text
                char_available = char_limit-len(output_text)
                if hashtag_limit and hashtag_count == hashtag_limit:
                    break
    return output_text
def conform_params_to_char_limits(input_params,characters,hash_symbol):
    max_chars = characters.get("limit")
    chars_available = max_chars
    url_len = 0
    url = input_params.get('url')
    if url:
        url_len = characters.get('url_len') or len(str(url))
    chars_available-= url_len
    via = input_params.get('via')
    if via:
        input_params['via'] = f"@{via}"
        chars_available-=1
    via_len = len(input_params['via'])
    chars_available-= via_len
    hashtags = input_params.get('hashtags')
    keywords_text = get_max_hashtags(hashtags,char_limit=chars_available,hashtag_limit=2)
    if keywords_text:
        chars_available-=1
    keywords_len = len(keywords_text)
    chars_available-= keywords_len
    text = input_params.get('text')
    text = elipse_if_over(text,chars_available)
    if text:
        chars_available-=1
    text_len = len(text)
    input_params['text'] = text
    chars_available-= text_len
    chars_available+=keywords_len
    keywords_text = get_max_hashtags(hashtags,chars_available,hash_symbol=hash_symbol)
    input_params['hashtags'] = keywords_text
    return input_params
def get_output_params(platform,text=None,url=None,via=None,hashtags=None):
    keywords = generate_keywords_list(hashtags)
    
    text = text or ''
    url = url or ''
    input_params = {"text":text,"via":via,"url":url,"hashtags":keywords}
    share_params = get_share_params(platform)
    params = share_params.get('params')
    characters = share_params.get('characters')
    hash_symbol = share_params.get('hash_symbol')
    output_params = {'text':''}
    input_params = conform_params_to_char_limits(input_params,characters,hash_symbol)
    for param,values in params.items():
        value,input_params = get_precise_param(values,input_params)
        output_params[param] = value
    text_output = create_output_text(input_params,hash_symbol)
    if output_params['text'] and text_output:
        output_params['text']+='\n'
    output_params['text']+=text_output
    return output_params
def create_post(platform,text=None,url=None,via=None,hashtags=None,output_params=None):
    output_params = output_params or get_output_params(platform,text=text,url=url,via=via,hashtags=hashtags)
    share_params = get_share_params(platform)
    hash_symbol = share_params.get('hash_symbol')
    output_text = create_output_text(output_params,hash_symbol)
    return output_text
def create_post_url(platform,text=None,url=None,via=None,hashtags=None,output_params=None):
    output_params = output_params or get_output_params(platform,text=text,url=url,via=via,hashtags=hashtags)
    share_params = get_share_params(platform)
    share_url = share_params.get('url')
    output_url = create_output_url(output_params,share_url)
    return output_url




from abstract_webtools import *
from abstract_ocr import download_pdf
from io import BytesIO
def get_content(soup, tag, attr_name, attr_values, content_key):
    """Extract content from a BeautifulSoup object based on tag and attributes."""
    if not soup:
        return None
    # Ensure attr_values is a list or single value
    attr_values = make_list(attr_values)
    for value in attr_values:
        # Find all tags matching the criteria
        elements = soup.find_all(tag, {attr_name: value})
        for element in elements:
            content = element.get(content_key)
            if content:
                return content
    return None
def analyze_text_for_keywords(soup: str, keywords: List[str], lines_per_section: int = 5):
    keywords = break_down_keywords(keywords)
    text = soup.text
    lines=[]
    # Split into individual lines
    if text:
        lines = text.replace('.','.\n').replace('\n\n','\n').split('\n')
        if not lines:
            return "", 0
    # Group lines into sections
    sections = []
    for i in range(0, len(lines), lines_per_section):
        section = '\n'.join(lines[i:i + lines_per_section])
        sections.append(section)

    def count_keywords(section: str) -> int:
        section_lower = section.lower()
        return sum(section_lower.count(kw) for kw in keywords)
    max_count = 0
    max_section_index = 0
    for i, section in enumerate(sections):
        if not section.strip():
            continue
        count = count_keywords(section)
        if count > max_count:
            max_count = count
            max_section_index = i

    if max_count == 0:
        return "", 0
    section_text = sections[max_section_index].strip()
    return section_text
def break_down_keyword(keyword):
    all_key_words = [keyword]
    if '-' in keyword:
        all_key_words.append(keyword.replace('-',''))
        all_key_words+=keyword.split('-')
    return all_key_words
def break_down_keywords(keywords):
    all_key_words=[]
    keywords = keywords or []
    if isinstance(keywords,str):
        keywords = [eatAll(keyword,[' ']) for keyword in keywords.splir(',') if keyword]
    for keyword in keywords:
        keyword_breakdown = break_down_keyword(keyword)
        keyword_break = make_list(keyword_breakdown)
        all_key_words+=keyword_break
    all_key_words = list(set(make_list(all_key_words)))
    return all_key_words
def create_keys(titles):
    keywords = []
    for title in titles:
        keywords+=title.split(' ')
    keywords = list(set(keywords))
    all_valid = list('abcdefghijklmnopqrstuvwxyz1234567890'+'abcdefghijklmnopqrstuvwxyz'.upper())
    invalid = ['and','the','of','at']
    all_new_words = []
    for word in keywords:
        chars = ''
        for char in word:
            if char not in all_valid:
                char = ' '
            chars+=char
        chars_spl = chars.split(' ')
        chars = [char for char in chars_spl if char and char.lower() not in invalid] 
        all_new_words+=chars
    return all_new_words
def url_in_sources(media,sources):
    url = media.get('url')
    for i,source in enumerate(sources):
        source_url = source.get('url')
        if url == source_url:
            for key,value in media.items():
                sources[i][key] = source.get(key,value) or value
            return sources
    sources.append(media)
    return sources
def get_all_keywords(directory):
    list_dir = os.listdir(directory)
    all_jsons = get_all_json_paths(directory)
    all_keywords = []
    for file_path in all_jsons:
        if not file_path.endswith('sources.json') and not file_path.endswith('images.json'):
            json_data = get_json_data(file_path)
            keywords = json_data.get('keywords') or json_data.get('keywords_str') or []
            if isinstance(keywords,str):
                keywords = [eatAll(keyword,[' ','\n','\t','#',',']) for keyword in keywords.split(',') if keyword]
            all_keywords+=keywords
    return all_keywords
def get_phrase(soup,string):
    soup = soup.find_all('head')
    text = str(soup)
    targets = []
    string_lower = string.lower()
    lines = text.replace('\n',' ').replace('> <','><').split('><')
    for line in lines:
        line_lower = line.lower()
        if string_lower in line_lower:
            target_text = ''
            if '>' in line_lower:
                target_text = line.split('>')[1].split('<')[0]
            elif '=' in line_lower:
                target_text = line.split(string)[0].split('=')[1].split(' ')
                target_text = ' '.join(target_text[:-1])
            if target_text:
                target_text = eatAll(target_text,[' ','\t','\n','',"'",'"'])
                targets.append(target_text)
    return list(set(targets))
def get_youtube_info(url):
    info = downloadvideo(url)
    return info



from .abstract_types import *
from typing import *
from abstract_utilities import *
import json
from pathlib import Path
def dirname_to_it(string,directory):
    if string in directory:
        while True:
            basename = os.path.basename(directory)
            if string in basename:
                return directory
            prior_dir = directory
            directory = os.path.dirname(directory)
            if directory == prior_dir:
                return
            print(directory)
def is_valid_path(path: Path):
    # Exclude paths that start with the GVFS mount point
    if str(path).startswith("/run/user/1000/gvfs/"):
        return False
    try:
        return path.is_dir()
    except OSError:
        return False
def find_src_dir(substr='src', directory=None):
    """
    Searches for a directory whose name contains `substr` by first checking
    the given directory and its ancestors. If not found, recursively searches
    within the directory tree.
    """
    base_dir = Path(directory) if directory else Path.cwd()
    
    # Check the base directory and its parents
    for parent in [base_dir] + list(base_dir.parents):
        if substr in parent.name:
            return parent
    
    # If not found, search recursively in the directory tree
    for path in base_dir.rglob('*'):
        if is_valid_path(path) and substr in path.name:
            return path
    return None
def get_domain_from_src(directory=None):
    src_dir = find_src_dir(directory=directory)
    directory = os.path.dirname(src_dir)
    domain  = os.path.basename(directory)
    domain,ext = os.path.splitext(domain)
    if not domain.startswith('http'):
        domain = f'https://{domain}'
    domain = f'{domain}{ext or ".com"}'
    return domain
class serverManager(metaclass=SingletonMeta):
    def __init__(self, src_dir=None,domain=None,imgs_dir=None,fb_id=None,directory=None):
        """Initialize the file directory manager with base paths."""
        self.src_dir = src_dir or find_src_dir(directory=directory)
        self.domain = domain or get_domain_from_src(directory=self.src_dir or directory)
        self.pages_dir = os.path.join(self.src_dir, "pages")
        self.json_dir = os.path.join(self.src_dir, "json_pages")
        self.contents_dir = os.path.join(self.src_dir, "contents")
        self.public_dir = os.path.join(self.src_dir, "../public")
        self.imgs_dir = imgs_dir or os.path.join(self.public_dir,'imgs')
        self.fb_id = fb_id
        self.ensure_directories()

    def ensure_directories(self) -> None:
        """Create necessary directories if they don't exist."""
        for directory in [self.pages_dir, self.json_dir, self.contents_dir, self.public_dir]:
            os.makedirs(directory, exist_ok=True)

    def join_path(self, *paths: str) -> str:
        """Join multiple path segments into a single path."""
        return os.path.join(*paths)

    def get_file_paths(self, directory: str, filename: str, is_index: bool = False) -> Tuple[str, str, str]:
        """Generate paths for JSON, page, and content files based on directory and filename."""
        json_path = self.join_path(self.json_dir, directory, f"{filename}.json") if directory else self.join_path(self.json_dir, f"{filename}.json")
        pages_path = self.join_path(self.pages_dir, "home", f"{filename}.tsx") if is_index else self.join_path(self.pages_dir, directory, f"{filename}.tsx")
        contents_path = self.join_path(self.contents_dir, directory, f"{filename}.md") if directory else self.join_path(self.contents_dir, f"{filename}.md")
        return json_path, pages_path, contents_path

    def read_json(self, json_path: str) -> dict:
        """Read and parse a JSON file."""
        try:
            with open(json_path, 'r', encoding='utf-8') as f:
                return json.load(f)
        except (FileNotFoundError, json.JSONDecodeError) as e:
            print(f"Error reading {json_path}: {e}")
            return {}

    def write_to_file(self, contents: str, file_path: str) -> None:
        """Write contents to a file, creating directories if needed."""
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(contents)

    def extract_content_to_md(self, json_data: dict, contents_path: str) -> Optional[str]:
        """Extract content from JSON and save it as a Markdown file."""
        content = json_data.get("content")
        if not content:
            print(f"No content found in JSON for {contents_path}")
            return None
        self.write_to_file(content.strip(), contents_path)
        return contents_path

    def update_json(self, json_path: str, content_file: str) -> None:
        """Update JSON to reference the content file instead of embedding content."""
        json_data = self.read_json(json_path)
        if "content" in json_data:
            del json_data["content"]
            relative_content_path = os.path.relpath(content_file, self.src_dir).replace("\\", "/")
            json_data["content_file"] = relative_content_path
            self.write_to_file(json.dumps(json_data, indent=4), json_path)

    def generate_tsx_template(self, filename: str, directory: str, meta: dict, is_index: bool = False) -> str:
        """Generate TSX template for a page."""
        add_imports = ""
        json_path, pages_path, contents_path = self.get_file_paths(directory, filename, is_index=True)
        pdfViewer = meta.get('pdfViewer') or 'false'
        dirname = os.path.basename(directory) if directory else "home"  # Use 'home' for index
        base_name = filename.replace('-', '_')
        page_name = f"Page_{base_name}" if base_name[0].isdigit() else f"{base_name.capitalize()}Page"
        sources_name = f"{page_name}Sources"
        return f"""import {{ GetStaticProps, NextPage }} from 'next';
import Head from 'next/head';
import path from 'path';
import fs from 'fs';
import {{ generateFullPageMetadata }} from '@MetaHead';
import {{ build_content }} from '@functions';
import {{ ImageData, PageData, SourceProps }} from '@interfaces';
import PageHeader from '@PageHeader';
import Body from '@Body';

const {sources_name}: NextPage<SourceProps> = ({{ metadataHTML, pageData, imageData }}) => {{
  return (
    <>
      <Head>
        <meta charSet="UTF-8" />
        <link rel="icon" href="/imgs/favicon.ico" type="image/x-icon"/>
        <meta property="og:image" content={{imageData['social_meta']['og:image']}}/>
        <meta property="og:url" content={{pageData.share_url}}/>
        <meta property="og:type" content="article"/>
        <meta property="og:title" content={{pageData.title}}/>
        <meta property="og:description" content={{pageData.description}}/>
        <meta property="fb:app_id" content="{self.fb_id}"/>
        <title>{{pageData.title}}</title>
        <div dangerouslySetInnerHTML={{{{ __html: metadataHTML }}}} />
      </Head>
      <PageHeader meta={{pageData}} baseUrl={{pageData.BASE_URL}} />
      <Body meta={{pageData}} pdfViewer={{pageData?.pdfViewer}} />
    </>
  );
}};

export const getStaticProps: GetStaticProps<SourceProps> = async () => {{
  try {{
    const filename = '{filename}';
    const jsonPath = '{json_path}'
    const contentsPath = '{contents_path}'
    const pagesPath ='{pages_path}'

    const jsonContents = fs.readFileSync(jsonPath, 'utf8');
    const pageData: PageData = JSON.parse(jsonContents);
    const thumbnailPath = path.join(pageData.thumbnail, 'info.json');
    
    const imageContents = fs.readFileSync(thumbnailPath, 'utf8');
    const imageData = JSON.parse(imageContents);
    pageData.imageData = imageData;

    const pageContents = fs.readFileSync(contentsPath, 'utf8');
    const pageContent = await build_content(pageContents, pageData, '{dirname if directory else ""}');
    pageData.content = pageContent;

    const {{ toHTML }} = generateFullPageMetadata(pageData, imageData);
    const metadataHTML = toHTML();

    return {{
      props: {{
        metadataHTML,
        pageData,
        imageData
      }},
    }};
  }} catch (error) {{
    console.error('Error in getStaticProps:', error);
    return {{
      notFound: true,
    }};
  }}
}};

export default {sources_name};
        """

    def process_directory(self, directory: str) -> None:
        """Process all JSON files in a directory."""
        json_directory = self.join_path(self.json_dir, directory)
        if not os.path.isdir(json_directory):
            print(f"Directory not found: {json_directory}")
            return

        for item in os.listdir(json_directory):
            filename, ext = os.path.splitext(item)
            if ext.lower() != ".json":
                continue

            is_index = (directory == "" and filename == "index")  # Check if it's the root index
            json_path, pages_path, contents_path = self.get_file_paths(directory, filename, is_index)
            json_data = self.read_json(json_path)

            # Extract content to Markdown
            content_file = self.extract_content_to_md(json_data, contents_path)
            if content_file:
                self.update_json(json_path, content_file)
    
            # Generate or update TSX page
            tsx_content = self.generate_tsx_template(filename, directory, json_data, is_index)
            self.write_to_file(tsx_content, pages_path)
            print(f"Processed: {json_path} -> {contents_path}, {pages_path}")

    def process_root_index(self) -> None:
        """Process the root index.json file separately."""
        json_path = self.join_path(self.json_dir, "index.json")
        if not os.path.exists(json_path):
            print(f"Root index.json not found: {json_path}")
            return

        filename = "index"
        directory = ""  # Empty directory for root-level index
        json_data = self.read_json(json_path)
        json_path, pages_path, contents_path = self.get_file_paths(directory, filename, is_index=True)

        # Extract content to Markdown
        content_file = self.extract_content_to_md(json_data, contents_path)
        if content_file:
            self.update_json(json_path, content_file)

        # Generate TSX page at /home/index.tsx
        tsx_content = self.generate_tsx_template(filename, directory, json_data, is_index=True)
        self.write_to_file(tsx_content, pages_path)
        print(f"Processed root index: {json_path} -> {contents_path}, {pages_path}")

    def process_all(self) -> None:
        """Process all directories in json_pages and the root index.json."""
        # Process root index.json first
        self.process_root_index()

        # Process all directories
        for directory in os.listdir(self.json_dir):
            if os.path.isdir(self.join_path(self.json_dir, directory)):
                self.process_directory(directory)



from abstract_utilities import *
from .abstract_webserver import *
from .get_image_json import *
import glob
BASE_URL= "https://thedailydialectics.com",
IMGS_DIR = '/var/www/thedailydialectics/public/imgs'
sample = {"BASE_URL": BASE_URL,
    "href": "/mems/micro-fluidics.json",
    "title": "Micro Fluidics System Diagram | The Daily Dialectics",
    "content_file": "contents/mems/micro-fluidics.md",
    "share_url": "https://thedailydialectics.com/mems/micro-fluidics",
    "description": "A detailed illustration of a micro fluidics system showcasing various components and their functions, highlighting the intricacies of fluid dynamics in miniatur",
    "keywords_str": "Micro Fluidics,Fluid Dynamics,Lab-on-a-Chip,Microfluidic Systems,CO2 Emissions Solutions",
    "thumbnail":"/var/www/thedailydialectics/public/imgs/mems/micro-fluidics"}
def clean_var(obj):
    obj = eatAll(obj,['#',' ','/','-',',','\t','\n'])
    return obj
def sanitize_dir_filename(directory,filename):
    basename = eatAll(os.path.basename(directory),'/')
    filename = eatAll(os.path.splitext(filename)[0],'/')
    return basename,filename
def get_href(directory,filename,data):
    basename,filename = sanitize_dir_filename(directory,filename)
    return f"/{basename}/{filename}.json"
def get_content_file(directory,filename,data):
    basename,filename = sanitize_dir_filename(directory,filename)
    return f"/{basename}/{filename}.md"
def get_share_url(directory,filename,data):
    basename,filename = sanitize_dir_filename(directory,filename)
    return f"{BASE_URL}/{basename}/{filename}"
def convert_keyword_to_string(keywords):
    if isinstance(keywords,list):
        keywords = [clean_var(keyword) for keyword in keywords if clean_var(keyword)]
        keywords = ','.join(keywords)
    return keywords or ''
def get_keywords(dirname,filename,data):
    keywords = convert_keyword_to_string(data.get('keyword_str') or [])
    keywords += ','+convert_keyword_to_string(data.get('keyword') or [])
    keywords = eatAll(keywords,',') 
    return keywords
def clear_all_thumbs(data):
    data_copy = data.copy()
    for key,value in data_copy.items():
        if 'thumbnail' in key:
            del data[key]
    return data
def get_media_from_thumbnail(thumbnail,media):
    for key,values in media.items():
        if thumbnail in values:
            return media
def get_json_directory():
    return '/var/www/thedailydialectics/src/json_pages'
def get_all_json_paths(directory):
    return glob.glob(os.path.join(directory, '**', '*.json'), recursive=True)
def conventionalize_jsons(directory):
    json_paths = get_all_json_paths(directory)
    for json_path in json_paths:
        dirname = os.path.dirname(json_path)
        dirname = os.path.basename(dirname)
        section = os.path.basename(dirname)
        basename = os.path.basename(json_path)
        filename,ext = os.path.splitext(basename)
        json_data  = get_json_data(json_path)
        new_data = {"BASE_URL": BASE_URL,'title':json_data.get('title'),'description':json_data.get('description')}
        new_data['href'] = get_href(dirname,filename,json_data)
        new_data['get_content_file'] = get_content_file(dirname,filename,json_data)
        new_data['share_url'] = get_share_url(dirname,filename,json_data)
        new_data['keyword_str'] = get_keywords(dirname,filename,json_data)
        new_data['media'] = json_data
        thumbnail = json_data.get('thumbnail')
        if not thumbnail:
            input(json_data)
        if not os.path.isdir(thumbnail):
            for media in new_data['media']:
                match = get_media_from_thumbnail(thumbnail,media)
                if match:
                    get_image_info(media=media)
            new_data['thumbnail'] = get_image_json(media, section=section)
        else:
            new_data['thumbnail'] = thumbnail
        safe_dump_to_file(new_data,json_path)
